@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2002, 2003, 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2002-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 * Author: Krzysztof Zaraska <kzaraska@student.uci.agh.edu.pl>
 *
@@ -88,6 +88,11 @@ static int db_run(prelude_plugin_instance_t *pi, idmef_message_t *message)
         if ( ret < 0 )
                 prelude_log(PRELUDE_LOG_WARN, "could not insert message into database: %s.\n", preludedb_strerror(ret));
 
+        if ( prelude_error_get_code(ret) == PRELUDEDB_ERROR_CONNECTION )
+                ret = MANAGER_REPORT_PLUGIN_FAILURE_GLOBAL;
+        else
+                ret = MANAGER_REPORT_PLUGIN_FAILURE_SINGLE;
+
         return ret;
 }
 
@@ -105,7 +110,7 @@ static void db_destroy(prelude_plugin_instance_t *pi, prelude_string_t *out)
 
         if ( plugin->file )
                 free(plugin->file);
-        
+
         if ( plugin->name )
                 free(plugin->name);
 
@@ -136,8 +141,8 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
         preludedb_sql_t *sql;
         preludedb_sql_settings_t *settings;
         db_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(pi);
-        
-        ret = preludedb_sql_settings_new(&settings);        
+
+        ret = preludedb_sql_settings_new(&settings);
         if ( ret < 0 )
                 return ret;
 
@@ -146,7 +151,7 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
 
         if ( plugin->file )
                 preludedb_sql_settings_set_file(settings, plugin->file);
-        
+
         if ( plugin->port )
                 preludedb_sql_settings_set_port(settings, plugin->port);
 
@@ -171,46 +176,46 @@ static int db_init(prelude_plugin_instance_t *pi, prelude_string_t *out)
         else {
                 ret = preludedb_sql_enable_query_logging(sql, (strcmp(plugin->log, "-") == 0) ? NULL : plugin->log);
                 if ( ret < 0 ) {
-			preludedb_sql_destroy(sql);
+                        preludedb_sql_destroy(sql);
                         prelude_string_sprintf(out, "could not enable queries logging with log file '%s': %s",
                                                plugin->log, preludedb_strerror(ret));
-			return ret;
-		}
-	}
+                        return ret;
+                }
+        }
 
         ret = preludedb_new(&db, sql, NULL, NULL, 0);
         if ( ret < 0 ) {
-		preludedb_sql_destroy(sql);
+                preludedb_sql_destroy(sql);
                 prelude_string_sprintf(out, "could not initialize libpreludedb: %s", preludedb_strerror(ret));
-		return ret;
-	}
+                return ret;
+        }
 
         if ( plugin->db )
                 preludedb_destroy(plugin->db);
-        
+
         plugin->db = db;
-        
+
         return 0;
 }
 
 
 
-static int db_activate(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context) 
+static int db_activate(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         db_plugin_t *new;
-        
+
         new = calloc(1, sizeof(*new));
         if ( ! new )
                 return prelude_error_from_errno(errno);
 
-	new->type = strdup(DEFAULT_DATABASE_TYPE);
-	if ( ! new->type ) {
-		free(new);
-		return prelude_error_from_errno(errno);
-	}
+        new->type = strdup(DEFAULT_DATABASE_TYPE);
+        if ( ! new->type ) {
+                free(new);
+                return prelude_error_from_errno(errno);
+        }
 
         prelude_plugin_instance_set_plugin_data(context, new);
-        
+
         return 0;
 }
 
@@ -220,35 +225,35 @@ static int db_activate(prelude_option_t *opt, const char *optarg, prelude_string
 int db_LTX_manager_plugin_init(prelude_plugin_entry_t *pe, void *rootopt)
 {
         int ret;
-	prelude_option_t *opt;
+        prelude_option_t *opt;
         static manager_report_plugin_t db_plugin;
         int hook = PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG|PRELUDE_OPTION_TYPE_WIDE;
-        
+
         ret = preludedb_init();
-	if ( ret < 0 ) {
+        if ( ret < 0 ) {
                 prelude_log(PRELUDE_LOG_ERR, "error initializing libpreludedb: %s", preludedb_strerror(ret));
                 return ret;
         }
-        
+
         ret = prelude_option_add(rootopt, &opt, hook, 0, "db", "Options for the libpreludedb plugin",
                                  PRELUDE_OPTION_ARGUMENT_OPTIONAL, db_activate, NULL);
         if ( opt < 0 )
                 return ret;
-                
+
         prelude_plugin_set_activation_option(pe, opt, db_init);
 
-	ret = prelude_option_add(opt, NULL, hook, 't', "type", "Type of database (mysql/pgsql)",
+        ret = prelude_option_add(opt, NULL, hook, 't', "type", "Type of database (mysql/pgsql)",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED, db_set_type, db_get_type);
         if ( ret < 0 )
                 return ret;
-        
-	ret = prelude_option_add(opt, NULL, hook, 'l', "log",
+
+        ret = prelude_option_add(opt, NULL, hook, 'l', "log",
                                  "Log all queries in a file, should be only used for debugging purpose",
                                  PRELUDE_OPTION_ARGUMENT_OPTIONAL, db_set_log, db_get_log);
         if ( ret < 0 )
                 return ret;
-        
-	ret = prelude_option_add(opt, NULL, hook, 'h', PRELUDEDB_SQL_SETTING_HOST,
+
+        ret = prelude_option_add(opt, NULL, hook, 'h', PRELUDEDB_SQL_SETTING_HOST,
                                  "The host where the database server is running (in case of client/server database)",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED,  db_set_host, db_get_host);
         if ( ret < 0 )
@@ -259,31 +264,31 @@ int db_LTX_manager_plugin_init(prelude_plugin_entry_t *pe, void *rootopt)
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED,  db_set_file, db_get_file);
         if ( ret < 0 )
                 return ret;
-        
+
         ret = prelude_option_add(opt, NULL, hook, 'p', PRELUDEDB_SQL_SETTING_PORT,
                                  "The port where the database server is listening (in case of client/server database)",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED, db_set_port, db_get_port);
         if ( ret < 0 )
                 return ret;
-        
-	ret = prelude_option_add(opt, NULL, hook, 'd', PRELUDEDB_SQL_SETTING_NAME,
+
+        ret = prelude_option_add(opt, NULL, hook, 'd', PRELUDEDB_SQL_SETTING_NAME,
                                  "The name of the database where the alerts will be stored",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED, db_set_name, db_get_name);
         if ( ret < 0 )
                 return ret;
-        
-	ret = prelude_option_add(opt, NULL, hook, 'u', PRELUDEDB_SQL_SETTING_USER,
+
+        ret = prelude_option_add(opt, NULL, hook, 'u', PRELUDEDB_SQL_SETTING_USER,
                                  "User of the database (in case of client/server database)",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED, db_set_user, db_get_user);
         if ( ret < 0 )
                 return ret;
-        
-	ret = prelude_option_add(opt, NULL, hook, 'P', PRELUDEDB_SQL_SETTING_PASS,
+
+        ret = prelude_option_add(opt, NULL, hook, 'P', PRELUDEDB_SQL_SETTING_PASS,
                                  "Password for the user (in case of client/server database)",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED, db_set_pass, db_get_pass);
         if ( ret < 0 )
                 return ret;
-        
+
         prelude_plugin_set_name(&db_plugin, "db");
         prelude_plugin_set_destroy_func(&db_plugin, db_destroy);
         manager_report_plugin_set_running_func(&db_plugin, db_run);
